@@ -34,7 +34,7 @@ from skimage.measure import label
 from csbdeep.utils import normalize
 from skimage import filters
 from skimage.util import random_noise
-
+from scipy.ndimage import distance_transform_edt
 
 globalthreshold = 0.01
 def SimplePrediction(x, UnetModel, StarModel, n_tiles = (2,2), UseProbability = True, min_size = 20):
@@ -174,7 +174,7 @@ n_tiles = (1,2,2), doMask = True, smartcorrection = None, threshold = 20, projec
         Mask[i,:] = remove_small_objects(Mask[i,:].astype('uint16'), min_size = min_size)
     
     SizedMask[:, :Mask.shape[1], :Mask.shape[2]] = Mask
-    
+    imwrite((UNETResults + Name+ '.tif' ) , SizedMask.astype('uint16')) 
 
     SmartSeeds, _, StarImage = STARPrediction3D(gaussian_filter(image,filtersize), StarModel,  n_tiles, MaskImage = Mask, UseProbability = UseProbability, smartcorrection = smartcorrection)
     #Upsample images back to original size
@@ -186,7 +186,7 @@ n_tiles = (1,2,2), doMask = True, smartcorrection = None, threshold = 20, projec
             
     imwrite((StarDistResults + Name+ '.tif' ) , StarImage.astype('uint16'))
     imwrite((SmartSeedsResults + Name+ '.tif' ) , SizedSmartSeeds.astype('uint16'))
-    imwrite((UNETResults + Name+ '.tif' ) , SizedMask.astype('uint16')) 
+    
     
         
     return SizedSmartSeeds, SizedMask    
@@ -232,7 +232,7 @@ def UNETPrediction3D(image, model, n_tiles, axis):
     Finalimage = label(Filled)
     Finalimage = fill_label_holes(Finalimage)
     Finalimage = relabel_sequential(Finalimage)[0]
-    
+    Finalimage = expand_labels(Finalimage,2)
           
     return Finalimage
 
@@ -254,17 +254,8 @@ def STARPrediction3D(image, model, n_tiles, MaskImage = None, smartcorrection =
     image = zero_pad_time(image, 64, 64)
     grid = copymodel.config.grid
 
-    try:
-         MidImage, details = model.predict_instances(image, n_tiles = n_tiles)
-         SmallProbability, SmallDistance = model.predict(image, n_tiles = n_tiles)
-
-    except:
-            conf = copymodel.config
-            Dummy = StarDist3D(conf)
-            overlap = Dummy._axes_tile_overlap('ZYX')
-            model._tile_overlap = [overlap]
-            MidImage, details = model.predict_instances(image, n_tiles = n_tiles)
-            SmallProbability, SmallDistance = model.predict(image, n_tiles = n_tiles)
+    MidImage, details = model.predict_instances(image, n_tiles = n_tiles)
+    SmallProbability, SmallDistance = model.predict(image, n_tiles = n_tiles)
 
     StarImage = MidImage[:image.shape[0],:shape[0],:shape[1]]
     SmallDistance = MaxProjectDist(SmallDistance, axis=-1)
@@ -290,7 +281,7 @@ def STARPrediction3D(image, model, n_tiles, MaskImage = None, smartcorrection =
           
     Watershed, Markers = WatershedwithMask3D(MaxProjectDistance.astype('uint16'), StarImage.astype('uint16'), MaskImage.astype('uint16'), grid )
     Watershed = fill_label_holes(Watershed.astype('uint16'))
-  
+    Watershed = expand_labels(Watershed,2)
        
        
 
@@ -317,54 +308,122 @@ def VetoRegions(Image, Zratio = 3):
 
    
     
+def WatershedwithMask3D(Image, Label,mask, grid): 
+    properties = measure.regionprops(Label, Image) 
+    binaryproperties = measure.regionprops(label(mask), Image) 
+    
+    
+    Coordinates = [prop.centroid for prop in properties] 
+    BinaryCoordinates = [prop.centroid for prop in binaryproperties]
+    
+    Binarybbox = [prop.bbox for prop in binaryproperties]
+    Coordinates = sorted(Coordinates , key=lambda k: [k[0], k[1], k[2]]) 
+    
+    if len(Binarybbox) > 0:    
+            for i in range(0, len(Binarybbox)):
+                
+                box = Binarybbox[i]
+                inside = [iou3D(box, star) for star in Coordinates]
+                
+                if not any(inside) :
+                         Coordinates.append(BinaryCoordinates[i])    
+                         
+    
+    Coordinates.append((0,0,0))
+   
 
-def WatershedwithMask3D(Image, Label, mask, grid):
-          #Image = ProbabilityMap of Stardist
-          #Label = Label segmentation image of Stardist
-          #Mask = U-Net predicted image post binarization
-          properties = measure.regionprops(Label, Image)
-          cord = np.array([prop.centroid for prop
-          in properties])
-          Unet_out, nb_labels = label(mask, return_num=True)
-          # Getting the set of labels where the Stardist
-          #centroids fall
-          intersection = Unet_out[tuple(cord.T)]
-          # Creating a mapping to remove the
-          #connected componnents (cc)
-          # from Unet that contain a centroid
-          #from Stardist.
-          # After the following 2 opperations,
-          # mapping is a 1d array where:
-          # mapping[i] -> 0 if the cc
-          #contains a Stardist barycenter
-          # mapping[i] -> i otherwise
-          mapping = np.arange(nb_labels+1,
-          dtype=Unet_out.dtype)
-          mapping[intersection] = 0
-          # Applying the mapping, masked_Unet only has
-          # connected components that do not contain
-          # a Stardist centroid
-          masked_Unet = mapping[Unet_out]
-          # Only the necessary centroids are computed,
-          # the bounding boxes does not have to be computed
-          binaryproperties = measure.regionprops(masked_Unet)
-          bin_cord = [prop.centroid for prop
-          in binaryproperties]
-          # Concatenating all the centroids together
-          # and proceeding as before
-          cord = np.vstack(([0, 0, 0], cord, bin_cord))
-          cord_int = np.round(cord).astype(int)
-          markers_raw = np.zeros_like(Image)
-          markers_raw[tuple(cord_int.T)] =
-          1 + np.arange(len(cord))
-          markers =
-          morphology.dilation(markers_raw,
-          morphology.ball(2))
-          watershedImage =
-          watershed(-Image, markers, mask)
-          return watershedImage, markers
+    Coordinates = np.asarray(Coordinates)
+    coordinates_int = np.round(Coordinates).astype(int) 
+    
+    markers_raw = np.zeros_like(Image) 
+    markers_raw[tuple(coordinates_int.T)] = 1 + np.arange(len(Coordinates)) 
+    markers = morphology.dilation(markers_raw.astype('uint16'), morphology.ball(2))
+    
+    watershedImage = watershed(-Image, markers, mask = mask.copy()) 
 
 
+    
+    return watershedImage, markers
+
+
+def expand_labels(label_image, distance=1):
+    """Expand labels in label image by ``distance`` pixels without overlapping.
+    Given a label image, ``expand_labels`` grows label regions (connected components)
+    outwards by up to ``distance`` pixels without overflowing into neighboring regions.
+    More specifically, each background pixel that is within Euclidean distance
+    of <= ``distance`` pixels of a connected component is assigned the label of that
+    connected component.
+    Where multiple connected components are within ``distance`` pixels of a background
+    pixel, the label value of the closest connected component will be assigned (see
+    Notes for the case of multiple labels at equal distance).
+    Parameters
+    ----------
+    label_image : ndarray of dtype int
+        label image
+    distance : float
+        Euclidean distance in pixels by which to grow the labels. Default is one.
+    Returns
+    -------
+    enlarged_labels : ndarray of dtype int
+        Labeled array, where all connected regions have been enlarged
+    Notes
+    -----
+    Where labels are spaced more than ``distance`` pixels are apart, this is
+    equivalent to a morphological dilation with a disc or hyperball of radius ``distance``.
+    However, in contrast to a morphological dilation, ``expand_labels`` will
+    not expand a label region into a neighboring region.  
+    This implementation of ``expand_labels`` is derived from CellProfiler [1]_, where
+    it is known as module "IdentifySecondaryObjects (Distance-N)" [2]_.
+    There is an important edge case when a pixel has the same distance to
+    multiple regions, as it is not defined which region expands into that
+    space. Here, the exact behavior depends on the upstream implementation
+    of ``scipy.ndimage.distance_transform_edt``.
+    See Also
+    --------
+    :func:`skimage.measure.label`, :func:`skimage.segmentation.watershed`, :func:`skimage.morphology.dilation`
+    References
+    ----------
+    .. [1] https://cellprofiler.org
+    .. [2] https://github.com/CellProfiler/CellProfiler/blob/082930ea95add7b72243a4fa3d39ae5145995e9c/cellprofiler/modules/identifysecondaryobjects.py#L559
+    Examples
+    --------
+    >>> labels = np.array([0, 1, 0, 0, 0, 0, 2])
+    >>> expand_labels(labels, distance=1)
+    array([1, 1, 1, 0, 0, 2, 2])
+    Labels will not overwrite each other:
+    >>> expand_labels(labels, distance=3)
+    array([1, 1, 1, 1, 2, 2, 2])
+    In case of ties, behavior is undefined, but currently resolves to the
+    label closest to ``(0,) * ndim`` in lexicographical order.
+    >>> labels_tied = np.array([0, 1, 0, 2, 0])
+    >>> expand_labels(labels_tied, 1)
+    array([1, 1, 1, 2, 2])
+    >>> labels2d = np.array(
+    ...     [[0, 1, 0, 0],
+    ...      [2, 0, 0, 0],
+    ...      [0, 3, 0, 0]]
+    ... )
+    >>> expand_labels(labels2d, 1)
+    array([[2, 1, 1, 0],
+           [2, 2, 0, 0],
+           [2, 3, 3, 0]])
+    """
+
+    distances, nearest_label_coords = distance_transform_edt(
+        label_image == 0, return_indices=True
+    )
+    labels_out = np.zeros_like(label_image)
+    dilate_mask = distances <= distance
+    # build the coordinates to find nearest labels,
+    # in contrast to [1] this implementation supports label arrays
+    # of any dimension
+    masked_nearest_label_coords = [
+        dimension_indices[dilate_mask]
+        for dimension_indices in nearest_label_coords
+    ]
+    nearest_labels = label_image[tuple(masked_nearest_label_coords)]
+    labels_out[dilate_mask] = nearest_labels
+    return labels_out
 
 
     
